#include <stdio.h>

///////////////////////////////////////////////////////////////////////////////
// ERROR CODE :
#define		SUCCESS						0				// Success
#define		ERR_TYPE					-10				// nType error
#define		ERR_OPENED					-20				// Already opened
#define		ERR_NO_PRINTER				-30				// There is not printer driver
#define		ERR_NO_MEMORY				-50				// No memory
#define		ERR_HANDLE					-60				// Invalid handle
#define		ERR_TIMEOUT					-70				// Ttime out
#define		ERR_ACCESS					-80				// cannot read/write
#define		ERR_PARAM					-90				// param error
#define		ERR_NOT_SUPPORT				-100			// not support
#define		ERR_EXIST					-210			// Already the same thing
#define		ERR_SS_NOT_EXIST			-340			// Status service not started
#define		ERR_SPL_NOT_EXIST			-350			// Spooler service not started
#define		ERR_RESET					-400			// Now printer is reseting
#define		ERR_LOCKED					-1000			// Already locked 

///////////////////////////////////////////////////////////////////////////////
// Status Bit :
#define		ESB_NO_RESPONSE				0x00000001UL	//No response
#define		ESB_DRAWER_KICK				0x00000004UL	//Drawer kick-out connector pin3 is HIGH
#define		ESB_OFF_LINE				0x00000008UL	//Off-line
#define		ESB_COVER_OPEN				0x00000020UL	//Cover is open

#define		ESB_AUTOCUTTER_ERR			0x00000800UL	//Auto cutter error
#define		ESB_AUTORECOVER_ERR			0x00004000UL	//Auto-Recoverable error

#define		ESB_RECEIPT_NEAR_END		0x00020000UL	//Receipt paper roll near-end
#define		ESB_RECEIPT_END				0x00080000UL	//Receipt paper roll end

#define		ESB_SPOOLER_IS_STOPPED		0x80000000UL	// The spooer is stopped

#define		ESB_NOW_PRINTING			0x00000080UL	// Now printing 

///////////////////////////////////////////////////////////////////////////////
// BiOpenMonPrinter() :

// Type
//#define		TYPE_PORT					1				// use port name
#define		TYPE_PRINTER				2				// use printer driver name

///////////////////////////////////////////////////////////////////////////////
// BiOpenDrawer() :

// Drawer Number
#define		DRAWER_1				1				// Drawer 1
#define		DRAWER_2				2				// Drawer 2

// Start Time
#define		PULSE_100			1				// 100 m sec
#define		PULSE_200			2				// 200 m sec
#define		PULSE_300			3				// 300 m sec
#define		PULSE_400			4				// 400 m sec
#define		PULSE_500			5				// 500 m sec
//#define		PULSE_600			6				// 600 m sec
//#define		PULSE_700			7				// 700 m sec
//#define		PULSE_800			8				// 800 m sec

//function address 
typedef INT (WINAPI *lpEiOpenMonPrinter)(int, LPSTR);
typedef INT (WINAPI *lpEiCloseMonPrinter)(int);

typedef INT (WINAPI *lpEiLockPrinter)(int, DWORD);
typedef INT (WINAPI *lpEiUnlockPrinter)(int);

//typedef INT (WINAPI *lpEiSetMonInterval)(int, WORD, WORD);
//typedef INT (WINAPI *lpEiSetMonEtherInterval)(int, WORD);

typedef INT (WINAPI *lpEiDirectIO)(int, BYTE, LPBYTE, LPBYTE, LPBYTE, DWORD, BOOL);
typedef INT (WINAPI *lpEiDirectIOEx)(int, DWORD, LPBYTE, LPDWORD, LPBYTE, DWORD, BOOL, BYTE);

typedef INT (WINAPI *lpEiResetPrinter)(int);

//typedef INT (WINAPI *lpEiCancelError)(int);
//typedef INT (WINAPI *lpEiGetType)(int, LPBYTE, LPBYTE, LPBYTE, LPBYTE);

typedef INT (WINAPI *lpEiGetStatus)(int, LPDWORD);
typedef INT (WINAPI *lpEiSetStatusBackFunction)(int, int (CALLBACK *pStatusCB)(DWORD));
typedef INT (WINAPI *lpEiSetStatusBackFunctionEx)(int, int (CALLBACK *pStatusCB)(DWORD, LPSTR));
typedef INT (WINAPI *lpEiSetStatusBackWnd)(int, HANDLE, LPDWORD);
typedef INT (WINAPI *lpEiCancelStatusBack)(int);

typedef INT (WINAPI *lpEiPowerOff)(int);
typedef INT (WINAPI *lpEiGetCounter)(int, WORD, LPDWORD);
typedef INT (WINAPI *lpEiResetCounter)(int, WORD);
typedef INT (WINAPI *lpEiGetPrnCapability)(int, BYTE, LPBYTE, LPBYTE);

//typedef INT (WINAPI *lpEiSendDataFile)(int, LPCSTR);
//typedef INT (WINAPI *lpEiDirectSendRead)(int, LPCSTR, LPCSTR, LPDWORD, LPBYTE, DWORD, BOOL);
//typedef INT (WINAPI *lpEiSetDefaultEchoTime)(BYTE, WORD);
//typedef INT (WINAPI *lpEiSetEtherEchoTime)(int, BYTE, WORD);

typedef INT (WINAPI *lpEiOpenDrawer)(int, BYTE, BYTE);

typedef int (WINAPI *pStatusCB)(DWORD);
typedef int (WINAPI *pStatusCBEx)(DWORD, LPSTR);

typedef INT (WINAPI *lpEmuDM_GetProperty)(PDEVMODEA ,BYTE , PBYTE, PDWORD);
typedef INT (WINAPI *lpEmuDM_SetProperty)(PDEVMODEA ,BYTE , PBYTE, DWORD);


class CEmuPrinterAPI 
{
protected:
	// Handle to a DLL
	HMODULE hLibHandle;

private:
	// Function pointers
	lpEiOpenMonPrinter			m_EiOpenMonPrinter;
	lpEiCloseMonPrinter			m_EiCloseMonPrinter;

	lpEiLockPrinter				m_EiLockPrinter;
	lpEiUnlockPrinter			m_EiUnlockPrinter;

//	lpEiSetMonInterval			m_EiSetMonInterval;
//	lpEiSetMonEtherInterval		m_EiSetMonEtherInterval;

	lpEiDirectIO				m_EiDirectIO;
	lpEiDirectIOEx				m_EiDirectIOEx;

	lpEiResetPrinter			m_EiResetPrinter;

//	lpEiCancelError				m_EiCancelError;
//	lpEiGetType					m_EiGetType;

	lpEiGetStatus				m_EiGetStatus;
	lpEiSetStatusBackFunction	m_EiSetStatusBackFunction;
	lpEiSetStatusBackFunctionEx	m_EiSetStatusBackFunctionEx;
	lpEiSetStatusBackWnd		m_ESetStatusBackWnd;
	lpEiCancelStatusBack		m_EiCancelStatusBack;

	lpEiPowerOff				m_EiPowerOff;

	lpEiGetCounter				m_EiGetCounter;
	lpEiResetCounter			m_EiResetCounter;
	lpEiGetPrnCapability		m_EiGetPrnCapability;

//	lpEiSendDataFile			m_EiSendDataFile;
//	lpEiDirectSendRead			m_EiDirectSendRead;
//	lpEiSetDefaultEchoTime		m_EiSetDefaultEchoTime;
//	lpEiSetEtherEchoTime		m_EiSetEtherEchoTime;

	lpEiOpenDrawer				m_EiOpenDrawer;
	lpEmuDM_GetProperty			m_EmuDM_GetProperty;
	lpEmuDM_SetProperty			m_EmuDM_SetProperty;

public:
	// Constructor
	CEmuPrinterAPI(void)
	{
		hLibHandle = NULL;

		m_EiOpenMonPrinter = NULL;
		m_EiCloseMonPrinter = NULL;

		m_EiLockPrinter = NULL;
		m_EiUnlockPrinter = NULL;

//		m_EiSetMonInterval = NULL;
//		m_EiSetMonEtherInterval = NULL;

		m_EiDirectIO = NULL;
		m_EiDirectIOEx = NULL;

		m_EiResetPrinter = NULL;

//		m_EiCancelError = NULL;
//		m_EiGetType = NULL;

		m_EiGetStatus = NULL;
		m_EiSetStatusBackFunction = NULL;
		m_EiSetStatusBackFunctionEx = NULL;
		m_ESetStatusBackWnd = NULL;
		m_EiCancelStatusBack = NULL;

		m_EiPowerOff = NULL;

		m_EiGetCounter = NULL;
		m_EiResetCounter = NULL;
		m_EiGetPrnCapability = NULL;

//		m_EiSendDataFile = NULL;
//		m_EiDirectSendRead = NULL;
//		m_EiSetDefaultEchoTime = NULL;
//		m_EiSetEtherEchoTime = NULL;

		m_EiOpenDrawer = NULL;

		m_EmuDM_GetProperty =NULL;
		m_EmuDM_SetProperty =NULL;
	};
	//Destructor
	virtual ~CEmuPrinterAPI()
	{
		if ( hLibHandle != NULL ) 
		{
			::FreeLibrary(hLibHandle);
		}
	};

	// Public API Functions. Must call Initialize() before anything else
	BOOL	Initialize()
	{
		hLibHandle = ::LoadLibrary("EmuPrinterAPI.dll");
		if ( hLibHandle != NULL ) 
		{
			// Load the necessary functions for this application
			m_EiOpenMonPrinter 	= (lpEiOpenMonPrinter)GetProcAddress(hLibHandle, "EiOpenMonPrinter");
			m_EiCloseMonPrinter	= (lpEiCloseMonPrinter)GetProcAddress(hLibHandle, "EiCloseMonPrinter");

			m_EiLockPrinter = (lpEiLockPrinter)GetProcAddress(hLibHandle, "EiLockPrinter");
			m_EiUnlockPrinter = (lpEiUnlockPrinter)GetProcAddress(hLibHandle, "EiUnlockPrinter");

//			m_EiSetMonInterval = (lpEiSetMonInterval)GetProcAddress(hLibHandle, "EiSetMonInterval");
//			m_EiSetMonEtherInterval = (lpEiSetMonEtherInterval)GetProcAddress(hLibHandle, "EiSetMonEtherInterval");

			m_EiDirectIO = (lpEiDirectIO)GetProcAddress(hLibHandle, "EiDirectIO");
			m_EiDirectIOEx = (lpEiDirectIOEx)GetProcAddress(hLibHandle, "EiDirectIOEx");

			m_EiResetPrinter = (lpEiResetPrinter)GetProcAddress(hLibHandle, "EiResetPrinter");

//			m_EiCancelError = (lpEiCancelError)GetProcAddress(hLibHandle, "EiCancelError");
//			m_EiGetType = (lpEiGetType)GetProcAddress(hLibHandle, "EiGetType");


			m_EiGetStatus = (lpEiGetStatus)GetProcAddress(hLibHandle, "EiGetStatus");
			m_EiSetStatusBackFunction = (lpEiSetStatusBackFunction)GetProcAddress(hLibHandle, "EiSetStatusBackFunction");
			m_EiSetStatusBackFunctionEx = (lpEiSetStatusBackFunctionEx)GetProcAddress(hLibHandle, "EiSetStatusBackFunctionEx");
			m_ESetStatusBackWnd = (lpEiSetStatusBackWnd)GetProcAddress(hLibHandle, "EiSetStatusBackWnd");
			m_EiCancelStatusBack = (lpEiCancelStatusBack)GetProcAddress(hLibHandle, "EiCancelStatusBack");

			m_EiPowerOff = (lpEiPowerOff)GetProcAddress(hLibHandle, "EiPowerOff");

			m_EiGetCounter = (lpEiGetCounter)GetProcAddress(hLibHandle, "EiGetCounter");
			m_EiResetCounter = (lpEiResetCounter)GetProcAddress(hLibHandle, "EiResetCounter");
			m_EiGetPrnCapability = (lpEiGetPrnCapability)GetProcAddress(hLibHandle, "EiGetPrnCapability");

//			m_EiSendDataFile = (lpEiSendDataFile)GetProcAddress(hLibHandle, "EiSendDataFile");
//			m_EiDirectSendRead = (lpEiDirectSendRead)GetProcAddress(hLibHandle, "EiDirectSendRead");
//			m_EiSetDefaultEchoTime = (lpEiSetDefaultEchoTime)GetProcAddress(hLibHandle, "EiSetDefaultEchoTime");
//			m_EiSetEtherEchoTime = (lpEiSetEtherEchoTime)GetProcAddress(hLibHandle, "EiSetEtherEchoTime");

			m_EiOpenDrawer = (lpEiOpenDrawer)GetProcAddress(hLibHandle, "EiOpenDrawer");

			m_EmuDM_GetProperty = (lpEmuDM_GetProperty)GetProcAddress(hLibHandle,"EmuDM_GetProperty");
			m_EmuDM_SetProperty = (lpEmuDM_SetProperty)GetProcAddress(hLibHandle,"EmuDM_SetProperty");

			if (m_EiOpenMonPrinter != NULL &&
				m_EiCloseMonPrinter!= NULL &&
	
				m_EiLockPrinter != NULL &&
				m_EiUnlockPrinter != NULL &&

//				m_EiSetMonInterval != NULL &&
//				m_EiSetMonEtherInterval != NULL &&

				m_EiDirectIO != NULL &&
				m_EiDirectIOEx != NULL &&

				m_EiResetPrinter != NULL &&

//				m_EiCancelError != NULL &&
//				m_EiGetType != NULL &&

				m_EiGetStatus != NULL &&
				m_EiSetStatusBackFunction != NULL &&
				m_EiSetStatusBackFunctionEx != NULL &&
				m_ESetStatusBackWnd != NULL &&
				m_EiCancelStatusBack != NULL &&

				m_EiPowerOff != NULL &&

				m_EiGetCounter != NULL &&
				m_EiResetCounter != NULL &&
				m_EiGetPrnCapability != NULL &&

//				m_EiSendDataFile != NULL &&
//				m_EiDirectSendRead != NULL &&
//				m_EiSetDefaultEchoTime != NULL &&
//				m_ESetEtherEchoTime != NULL && 
				m_EiOpenDrawer != NULL &&
				m_EmuDM_GetProperty != NULL &&
				m_EmuDM_SetProperty != NULL)
	
				return TRUE;
			else	 
				return FALSE;
		}
		else 
			return FALSE;
	};

	int EiOpenMonPrinter(int i_type,PCHAR i_prt)
	{
		if (  m_EiOpenMonPrinter!=NULL ) 
			return m_EiOpenMonPrinter(i_type, i_prt);
		return ERR_NO_PRINTER;
	};
	int EiCloseMonPrinter(int i_hdl)
	{
		if (  m_EiCloseMonPrinter!=NULL ) 
			return m_EiCloseMonPrinter(i_hdl);
		return ERR_NO_PRINTER;

	}
	
	int EiLockPrinter(int i_hdl,DWORD i_timeout)
	{
		if (  m_EiLockPrinter!=NULL ) 
			return m_EiLockPrinter(i_hdl,i_timeout);
		return ERR_NO_PRINTER;
	}
	int EiUnlockPrinter(int i_hdl)
	{
		if (  m_EiUnlockPrinter!=NULL ) 
			return m_EiUnlockPrinter(i_hdl);
		return ERR_NO_PRINTER;
	}

//	int EiSetMonInterval(int i_hdl, WORD wUnused, WORD wInterval)
//	{
//		if (  m_EiSetMonInterval!=NULL ) 
//			return m_EiSetMonInterval(i_hdl, wUnused, wInterval);
//		return ERR_NO_PRINTER;
//	}
//	int EiSetMonEtherInterval(int i_hdl, WORD wEtherInterval)
//	{
//		if (  m_EiSetMonEtherInterval!=NULL ) 
//			return m_EiSetMonEtherInterval(i_hdl, wEtherInterval);
//		return ERR_NO_PRINTER;
//	}

	int EiDirectIO(int i_hdl, BYTE i_wlen, PBYTE i_wcmd,
               PBYTE io_rlen, PBYTE o_rbuf,DWORD i_timeout,BOOL i_nt)
	{
		if (  m_EiDirectIO!=NULL ) 
			return m_EiDirectIO(i_hdl, i_wlen, i_wcmd, io_rlen, o_rbuf, i_timeout, i_nt);
		return ERR_NO_PRINTER;
	}
	int EiDirectIOEx(int i_hdl, BYTE i_wlen, PBYTE i_wcmd,
               PDWORD io_rlen, PBYTE o_rbuf,DWORD i_timeout,BOOL i_nt,BYTE i_op)
	{
		if (  m_EiDirectIOEx!=NULL ) 
			return m_EiDirectIOEx(i_hdl, i_wlen, i_wcmd, io_rlen, o_rbuf, i_timeout, i_nt,i_op);
		return ERR_NO_PRINTER;
	}

	int EiResetPrinter(int i_hdl)
	{
		if (  m_EiResetPrinter!=NULL ) 
			return m_EiResetPrinter(i_hdl);
		return ERR_NO_PRINTER;
	}

//	int EiCancelError(int i_hdl)
//	{
//		if (  m_EiCancelError!=NULL ) 
//			return m_EiCancelError(i_hdl);
//		return ERR_NO_PRINTER;
//	}
//	int EiGetType(int i_hdl,LPBYTE typeID, LPBYTE font, LPBYTE exrom,LPBYTE special)
//	{
//		if (  m_EiGetType!=NULL ) 
//			return m_EiGetType(i_hdl, typeID, font, exrom, special);
//		return ERR_NO_PRINTER;
//	}

	int EiGetStatus(int i_hdl,PDWORD o_status)
	{
		if (  m_EiGetStatus!=NULL ) 
			return m_EiGetStatus(i_hdl,o_status);
		return ERR_NO_PRINTER;
	}
	int EiSetStatusBackFunction(int i_hdl,pStatusCB i_stcb)
	{
		if (  m_EiSetStatusBackFunction!=NULL ) 
			return m_EiSetStatusBackFunction(i_hdl,i_stcb);
		return ERR_NO_PRINTER;
	}
	int EiSetStatusBackFunctionEx(int i_hdl,pStatusCBEx i_stcb)
	{
		if (  m_EiSetStatusBackFunctionEx!=NULL ) 
			return m_EiSetStatusBackFunctionEx(i_hdl,i_stcb);
		return ERR_NO_PRINTER;
	}
	int EiSetStatusBackWnd(int i_hdl,HANDLE i_Wnd, PDWORD i_status)
	{
		if (  m_ESetStatusBackWnd!=NULL ) 
			return m_ESetStatusBackWnd(i_hdl,i_Wnd ,i_status);
		return ERR_NO_PRINTER;
	}
	int EiCancelStatusBack(int i_hdl)
	{
		if (  m_EiCancelStatusBack!=NULL ) 
			return m_EiCancelStatusBack(i_hdl);
		return ERR_NO_PRINTER;
	}

	int EiPowerOff(int i_hdl)
	{
		if (  m_EiPowerOff!=NULL ) 
			return m_EiPowerOff(i_hdl);
		return ERR_NO_PRINTER;
	}

	int EiGetCounter(int i_hdl,WORD i_readno,PDWORD o_readcounter)
	{
		if (  m_EiGetCounter!=NULL ) 
			return m_EiGetCounter(i_hdl, i_readno, o_readcounter);
		return ERR_NO_PRINTER;
	}
	int EiResetCounter(int i_hdl,WORD i_readno)
	{
		if (  m_EiResetCounter!=NULL ) 
			return m_EiResetCounter(i_hdl, i_readno);
		return ERR_NO_PRINTER;
	}
	int EiGetPrnCapability(int i_hdl,BYTE i_id,PBYTE io_datsize,PBYTE o_dat)
	{
		if (  m_EiGetPrnCapability!=NULL ) 
			return m_EiGetPrnCapability(i_hdl, i_id, io_datsize, o_dat);
		return ERR_NO_PRINTER;
	}

//	int EiSendDataFile(int i_hdl,LPCSTR lpcFileName)
//	{
//		if (  m_EiSendDataFile!=NULL ) 
//			return m_EiSendDataFile(i_hdl,lpcFileName);
//		return ERR_NO_PRINTER;
//	}
//	int EiDirectSendRead(int i_hdl,LPCSTR lpcCmdName,
//                                LPCSTR lpcReadName,
//                                LPDWORD lpdwLength, LPBYTE lpbReadBuf,
//                                DWORD dwTimeout, BOOL blNullTerminate)
//	{
//		if (  m_EiDirectSendRead!=NULL ) 
//			return m_EiDirectSendRead(i_hdl, lpcCmdName, lpcReadName, lpdwLength, lpbReadBuf,dwTimeout,blNullTerminate);
//		return ERR_NO_PRINTER;
//	}
//	int EiSetDefaultEchoTime(BYTE Count, WORD Timeout)
//	{
//		if (  m_EiSetDefaultEchoTime!=NULL ) 
//			return m_EiSetDefaultEchoTime(Count, Timeout);
//		return ERR_NO_PRINTER;
//	}
//	int EiSetEtherEchoTime(BYTE Count, WORD Timeout)
//	{
//		if (  m_ESetEtherEchoTime!=NULL ) 
//			return m_ESetEtherEchoTime(Count, Timeout);
//		return ERR_NO_PRINTER;
//	}

	int EiOpenDrawer(int i_hdl,BYTE i_drawer, BYTE i_pulse)
	{
		if (  m_EiOpenDrawer!=NULL ) 
			return m_EiOpenDrawer(i_hdl, i_drawer, i_pulse);
		return ERR_NO_PRINTER;
	}

	int EmuDM_GetProperty(PDEVMODEA i_devmode,BYTE i_pid, PBYTE o_dat, PDWORD io_size)
	{
		if(m_EmuDM_GetProperty != NULL)
			return m_EmuDM_GetProperty(i_devmode,i_pid,o_dat,io_size);
		return ERR_NO_PRINTER;
	}
	int EmuDM_SetProperty(PDEVMODEA i_devmode,BYTE i_pid, PBYTE i_dat, DWORD i_size)
	{
		if(m_EmuDM_SetProperty != NULL)
			return m_EmuDM_SetProperty(i_devmode,i_pid,i_dat,i_size);
		return ERR_NO_PRINTER;
	}

};